/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package dialogs;

import java.util.ResourceBundle;


import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import core.Loader;

import utils.StockIcon;

/**
 * Instances of this class show the difference
 * between two given images in a new window.
 */
public class BeforeAndAfter {

	/**
	 * The shell used by this dialog
	 */
	private Shell sShell = null;

	/**
	 * Composite used to show the left (before) image
	 */
	private Composite compositeBefore = null;

	/**
	 * Composite used to show the right (after) image
	 */
	private Composite compositeAfter = null;

	/**
	 * Composite used to show the buttons to the bottom of the window
	 */
	private Composite compositeBottom = null;

	/**
	 * The close button
	 */
	private Button buttonClose = null;

	//left and right labels used to show the strings "before" and "after"
	private Label labelBefore = null;
	private Label labelAfter = null;

	/**
	 * This dialog's Shell parent
	 */
	private final Shell parent;

	//the two images created from the given image data
	private final Image beforeImage;
	private final Image afterImage;

	/**
	 * messages that are to be used by the entire GUI
	 */
	private ResourceBundle messages;

	//TODO check for the exceptions in the comment
	/**
	 * Creates a new instance of this class given it's parent Shell window
	 * and the two image data used to create this dialog's images
	 * @param parent the dialog's parent
	 * @param beforeImageData the image to be shown in the "before" composite
	 * @param afterImageData the image to be shown in the "after" composite
	 *
	 * @exception IllegalArgumentException <ul>
	 *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
	 * </ul>
	 * @exception SWTException <ul>
	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
	 *    <li>ERROR_INVALID_SUBCLASS - if this class is not an allowed subclass</li>
	 * </ul>
	 */
	public BeforeAndAfter(Shell parent, ImageData beforeImageData, ImageData afterImageData) {
		this.parent = parent;
		this.beforeImage = new Image(Display.getCurrent(), beforeImageData);
		this.afterImage = new Image(Display.getCurrent(), afterImageData);
		messages = Loader.getMessages();
		createSShell();
	}

	/**
	 * This method initializes sShell
	 */
	private void createSShell() {

		//grid layout used by the shell
		GridLayout gridLayoutShell = new GridLayout();
		gridLayoutShell.numColumns = 2;
		sShell = new Shell(parent, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
		sShell.setText(messages.getString("BEFORE_AND_AFTER_WINDOW_TITLE"));
		sShell.setLayout(gridLayoutShell);

		//the two (before and after) labels in the shell
		labelBefore = new Label(sShell, SWT.NONE);
		labelBefore.setText(messages.getString("BEFORE_AND_AFTER_WINDOW_BEFORE_LABEL"));
		labelAfter = new Label(sShell, SWT.NONE);
		labelAfter.setText(messages.getString("BEFORE_AND_AFTER_WINDOW_AFTER_LABEL"));

		//create the 3 composites
		createCompositeBefore();
		createCompositeAfter();
		createCompositeBottom();

		sShell.pack();
		sShell.open();

		//dispose the two created images on disposing this dialog
		sShell.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				beforeImage.dispose();
				afterImage.dispose();
			}
		});
	}

	/**
	 * This method initializes compositeBefore
	 */
	private void createCompositeBefore() {
		GridData gridDataComposite = new GridData();
		//TODO -2 values are because SWT.BORDER adds those pixels in composite. why!?
		gridDataComposite.widthHint = beforeImage.getBounds().width - 2;
		gridDataComposite.heightHint = beforeImage.getBounds().height - 2;
		compositeBefore = new Composite(sShell, SWT.BORDER);
		compositeBefore.setLayout(new GridLayout());
		compositeBefore.setLayoutData(gridDataComposite);
		compositeBefore.setBackgroundImage(beforeImage);
	}

	/**
	 * This method initializes compositeAfter
	 */
	private void createCompositeAfter() {
		GridData gridDataComposite = new GridData();
		//TODO -2 values are because SWT.BORDER adds those pixels in composite. why!?
		gridDataComposite.widthHint = afterImage.getBounds().width - 2;
		gridDataComposite.heightHint = afterImage.getBounds().height - 2;
		compositeAfter = new Composite(sShell, SWT.BORDER);
		compositeAfter.setLayout(new GridLayout());
		compositeAfter.setLayoutData(gridDataComposite);
		compositeAfter.setBackgroundImage(afterImage);
	}

	/**
	 * This method initializes compositeBottom
	 */
	private void createCompositeBottom() {
		GridData gridDataButtonClose = new GridData();
		GridLayout gridLayoutComposite = new GridLayout();
		gridLayoutComposite.marginHeight = 1;
		gridLayoutComposite.marginWidth = 1;
		GridData gridDataComposite = new GridData();
		gridDataComposite.horizontalSpan = 2;
		gridDataComposite.horizontalAlignment = org.eclipse.swt.layout.GridData.END;
		compositeBottom = new Composite(sShell, SWT.NONE);
		compositeBottom.setLayoutData(gridDataComposite);
		compositeBottom.setLayout(gridLayoutComposite);
		buttonClose = new Button(compositeBottom, SWT.NONE);
		buttonClose.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_CLOSE, StockIcon.SIZE_LARGE_TOOLBAR, true)));
		buttonClose.setLayoutData(gridDataButtonClose);
		buttonClose.setText(messages.getString("BEFORE_AND_AFTER_WINDOW_CLOSE_BUTTON"));
		buttonClose.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				sShell.close();
			}
		});
		gridDataButtonClose.widthHint = buttonClose.computeSize(-1, -1).x + 10;
	}
}